package util;

import java.util.Collections;
import java.util.Map;
import java.util.concurrent.Callable;

import org.apache.commons.collections4.map.LRUMap;
import org.springframework.cache.support.AbstractValueAdaptingCache;
import org.springframework.lang.Nullable;

/**
 * {@link org.springframework.cache.Cache Cache} implementation that stores cache items in LRU map.
 */
public class LRUMapCache extends AbstractValueAdaptingCache {

    private final String name;

    private final Map<Object, Object> store;

    /**
     * Create a new cache with the specified name and maximum size.
     */
    public LRUMapCache(String name, int maxSize) {
        super(false);

        this.name = name;
        this.store = Collections.synchronizedMap(new LRUMap<>(maxSize));
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public Map<Object, Object> getNativeCache() {
        return store;
    }

    @Override
    protected Object lookup(Object key) {
        return store.get(key);
    }

    @SuppressWarnings("unchecked")
    @Override
    @Nullable
    public <T> T get(Object key, Callable<T> valueLoader) {
        return (T) fromStoreValue(store.computeIfAbsent(key, r -> {
            try {
                return toStoreValue(valueLoader.call());
            }
            catch (Throwable ex) {
                throw new ValueRetrievalException(key, valueLoader, ex);
            }
        }));
    }

    @Override
    public void put(Object key, @Nullable Object value) {
        store.put(key, toStoreValue(value));
    }

    @Override
    @Nullable
    public ValueWrapper putIfAbsent(Object key, @Nullable Object value) {
        Object existing = this.store.putIfAbsent(key, toStoreValue(value));
        return toValueWrapper(existing);
    }

    @Override
    public void evict(Object key) {
        store.remove(key);
    }

    @Override
    public void clear() {
        this.store.clear();
    }
}
